@@ -1,18 +1,21 @@
 import datetime
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, Generator, List, Union
 
 from konan_sdk.endpoints.base_endpoint import (
+    KonanBaseDeploymentPredictionsEndpoint,
     KonanBaseEndpoint,
     KonanBaseGenericDeploymentsEndpoint,
     KonanBaseDeploymentEndpoint,
     KonanBaseDeploymentGenericModelsEndpoint,
     KonanBaseModelEndpoint,
+    KonanBaseLoginEndpoint,
     KonanEndpointOperationEnum,
 )
 from konan_sdk.endpoints.interfaces import (
     KonanEndpointRequest,
     KonanEndpointResponse,
 )
+from konan_sdk.endpoints.mixins import KonanPaginatedEndpointMixin
 from konan_sdk.konan_metrics import (
     KonanBaseMetric,
     KonanCustomMetric,
@@ -29,7 +32,7 @@ from konan_sdk.konan_types import (
     KonanModelCreationRequest,
     KonanModelState,
     KonanModel,
-    KonanTokens,
+    KonanProjectCreationRequest,
     KonanPrediction,
     KonanFeedbackSubmission, KonanFeedbackStatus, KonanFeedbacksResult,
     KonanTimeWindow,
@@ -37,18 +40,31 @@ from konan_sdk.konan_types import (
 from konan_sdk.konan_utils.enums import string_to_konan_enum
 
 
-class LoginEndpoint(KonanBaseEndpoint[KonanCredentials, KonanTokens]):
+class APIKeyLoginEndpoint(KonanBaseLoginEndpoint[str]):
+    @property
+    def name(self) -> str:
+        return 'login-api-key'
+
+    @property
+    def endpoint_path(self) -> str:
+        return super().endpoint_path + "/api_key/"
+
+    def prepare_request(
+        self, request_object: str
+    ) -> KonanEndpointRequest:
+        return KonanEndpointRequest(json={
+            'api_key': request_object,
+        })
+
+
+class LoginEndpoint(KonanBaseLoginEndpoint[KonanCredentials]):
     @property
     def name(self) -> str:
         return 'login'
 
     @property
     def endpoint_path(self) -> str:
-        return '/api/auth/login/'
-
-    @property
-    def endpoint_operation(self) -> KonanEndpointOperationEnum:
-        return KonanEndpointOperationEnum.POST
+        return super().endpoint_path + "/login/"
 
     def prepare_request(
         self, request_object: KonanCredentials
@@ -58,14 +74,6 @@ class LoginEndpoint(KonanBaseEndpoint[KonanCredentials, KonanTokens]):
             'password': request_object.password
         })
 
-    def process_response(
-        self, endpoint_response: KonanEndpointResponse
-    ) -> KonanTokens:
-        return KonanTokens(
-            endpoint_response.json['access'],
-            endpoint_response.json['refresh']
-        )
-
 
 class RefreshTokenEndpoint(KonanBaseEndpoint[str, str]):
     @property
@@ -233,14 +241,17 @@ class CreateDeploymentEndpoint(
     def prepare_request(
         self, request_object: KonanDeploymentCreationRequest
     ) -> KonanEndpointRequest:
-        return KonanEndpointRequest(json={
+        request_dict = {
             'deployment_name': request_object.name,
             'model_name': request_object.model_creation_request.name,
-            'docker_username': request_object.model_creation_request.docker_credentials.username,
-            'docker_password': request_object.model_creation_request.docker_credentials.password,
             'image_url': request_object.model_creation_request.docker_image.url,
             'exposed_port': request_object.model_creation_request.docker_image.exposed_port,
-        })
+        }
+        if request_object.model_creation_request.docker_credentials is not None:
+            request_dict['docker_username'] = request_object.model_creation_request.docker_credentials.username
+            request_dict['docker_password'] = request_object.model_creation_request.docker_credentials.password
+
+        return KonanEndpointRequest(json=request_dict)
 
     def process_response(
         self, endpoint_response: KonanEndpointResponse
@@ -274,6 +285,47 @@ class CreateDeploymentEndpoint(
         )
 
 
+class CreateProjectEndpoint(
+    KonanBaseGenericDeploymentsEndpoint[
+        KonanProjectCreationRequest,
+        KonanDeployment
+    ]
+):
+    @property
+    def name(self) -> str:
+        return 'create-project'
+
+    @property
+    def endpoint_path(self) -> str:
+        return super().endpoint_path + '/'
+
+    @property
+    def endpoint_operation(self) -> KonanEndpointOperationEnum:
+        return KonanEndpointOperationEnum.POST
+
+    def prepare_request(
+        self, request_object: KonanProjectCreationRequest
+    ) -> KonanEndpointRequest:
+        request_dict = {
+            'name': request_object.name,
+        }
+        if request_object.description:
+            request_dict['description'] = request_object.description
+
+        return KonanEndpointRequest(json=request_dict)
+
+    def process_response(
+        self, endpoint_response: KonanEndpointResponse
+    ) -> KonanDeployment:
+        return KonanDeployment(
+            endpoint_response.json['uuid'],
+            endpoint_response.json['name'],
+            datetime.datetime.fromisoformat(
+                endpoint_response.json['created_at'],
+            )
+        )
+
+
 class CreateModelEndpoint(
     KonanBaseDeploymentGenericModelsEndpoint[
         KonanModelCreationRequest,
@@ -295,13 +347,17 @@ class CreateModelEndpoint(
     def prepare_request(
         self, request_object: KonanModelCreationRequest
     ) -> KonanEndpointRequest:
-        return KonanEndpointRequest(json={
+        request_dict = {
             'name': request_object.name,
-            'docker_username': request_object.docker_credentials.username,
-            'docker_password': request_object.docker_credentials.password,
             'image_url': request_object.docker_image.url,
             'exposed_port': request_object.docker_image.exposed_port,
-        })
+            'state': request_object.state.value,
+        }
+        if request_object.docker_credentials is not None:
+            request_dict['docker_username'] = request_object.docker_credentials.username
+            request_dict['docker_password'] = request_object.docker_credentials.password
+
+        return KonanEndpointRequest(json=request_dict)
 
     def process_response(
         self, endpoint_response: KonanEndpointResponse
@@ -481,3 +537,39 @@ class DeleteDeployment(
         self, endpoint_response: KonanEndpointResponse
     ) -> bool:
         return True
+
+
+class GetPredictionsEndpoint(
+    KonanPaginatedEndpointMixin[KonanTimeWindow, KonanPrediction],
+    KonanBaseDeploymentPredictionsEndpoint[
+        KonanTimeWindow,
+        Generator[List[KonanPrediction], None, None],
+    ]
+):
+    @property
+    def name(self) -> str:
+        return 'get-predictions'
+
+    @property
+    def endpoint_path(self) -> str:
+        return super().endpoint_path + '/'
+
+    def prepare_request(
+        self, request_object: KonanTimeWindow,
+    ) -> KonanEndpointRequest:
+        return KonanEndpointRequest(params={
+            'start_time': request_object.start_time.isoformat(),
+            'end_time': request_object.end_time.isoformat(),
+        })
+
+    def _process_page(
+        self, results: List[Dict[str, Any]]
+    ) -> List[KonanPrediction]:
+        return [
+            KonanPrediction(
+                uuid=prediction['uuid'],
+                output=prediction['mls_output_json'],
+                features=prediction.get('features_json'),
+                feedback=prediction.get('feedback'),
+            ) for prediction in results
+        ]
