@@ -1,29 +1,36 @@
 import datetime
 import sys
+import deprecated
 from loguru import logger
 from typing import (
     Dict,
+    Generator,
     List,
     Optional,
     Tuple,
     Union
 )
 
-from konan_sdk.auth import KonanAuth
+from requests import HTTPError
+
+from konan_sdk.auth import KonanAPIKeyAuth, KonanAuth
 from konan_sdk.endpoints.konan_endpoints import (
     CreateDeploymentEndpoint,
     CreateModelEndpoint,
+    CreateProjectEndpoint,
     DeleteDeployment,
     DeleteModelEndpoint,
     EvaluateEndpoint,
     FeedbackEndpoint,
     GetModelsEndpoint,
+    GetPredictionsEndpoint,
     PredictionEndpoint,
     SwitchLiveModelEndpoint,
     SwitchNonLiveModelEndpoint,
 )
 from konan_sdk.konan_metrics import KonanBaseMetric
 from konan_sdk.konan_types import (
+    KonanDeployment,
     KonanDeploymentCreationRequest,
     KonanDeploymentCreationResponse,
     KonanDockerCredentials,
@@ -34,6 +41,8 @@ from konan_sdk.konan_types import (
     KonanModel,
     KonanModelCreationRequest,
     KonanModelState,
+    KonanPrediction,
+    KonanProjectCreationRequest,
     KonanTimeWindow,
 )
 from konan_sdk.konan_utils.models import (
@@ -58,21 +67,64 @@ class KonanSDK:
             logger.remove()
             logger.add(sys.stderr, level="INFO")
 
-    def login(self, email: str, password: str) -> None:
-        """Login to Konan with user credentials
+    @deprecated.deprecated(
+        reason='Password-based authentication will be removed in a future release.',
+        version='1.4.0',
+        category=DeprecationWarning,
+    )
+    def login(self, email: str = None, password: str = None, api_key: str = None) -> None:
+        """Login to Konan with _either_ email + password credentails _or_ an API Key.
 
-        :param email: email of registered user
-        :type email: str
-        :param password: password of registered user
-        :type password: str
+        At least one of the two authentication methods *must* be passed.
+        If both methods are passed, API Key authentication is used.
+
+        :param email: email of registered user, defaults to None
+        :type email: str, optional
+        :param password: password of registered user, defaults to None
+        :type password: str, optional
+        :param api_key: API Key of registered user, defaults to None
+        :type api_key: str, optional
         """
-        self.auth = KonanAuth(self.auth_url, email, password)
+        if api_key is None:
+            if email is None or password is None:
+                raise ValueError("Parameters for at least one authentication method must be passed")
+            self.auth = KonanAuth(auth_url=self.auth_url, email=email, password=password)
+        else:
+            self.auth = KonanAPIKeyAuth(auth_url=self.auth_url, api_key=api_key)
+
         self.auth.login()
 
-    def create_deployment(
+    def _create_project(self, name: str, description: str = None) -> KonanDeployment:
+        """Call the create project function
+
+        :param name: name of the project to create
+        :type name: str
+        :param description: description of the project to create, defaults to None
+        :type description: str, optional
+        :return: created Konan project
+        :rtype: KonanDeployment
+        """
+        # check user performed login
+        self.auth._post_login_checks()
+
+        # Check if access token is valid and retrieve a new one if needed
+        self.auth.auto_refresh_token()
+
+        deployment = CreateProjectEndpoint(
+            self.api_url,
+            user=self.auth.user
+        ).request(
+            KonanProjectCreationRequest(
+                name=name,
+                description=description,
+            )
+        )
+        return deployment
+
+    def _create_deployment(
         self,
         name: str,
-        docker_credentials: KonanDockerCredentials,
+        docker_credentials: Optional[KonanDockerCredentials],
         docker_image: KonanDockerImage,
         model_name: str = None,
     ) -> KonanDeploymentCreationResponse:
@@ -81,7 +133,7 @@ class KonanSDK:
         :param name: name of the deployment to create
         :type name: str
         :param docker_credentials: credentials for the docker registry to use
-        :type docker_credentials: KonanDockerCredentials
+        :type docker_credentials: Optional[KonanDockerCredentials]
         :param docker_image: docker image information
         :type docker_image: KonanDockerImage
         :param model_name: name of the live model to create, defaults to None
@@ -104,22 +156,81 @@ class KonanSDK:
             user=self.auth.user
         ).request(
             KonanDeploymentCreationRequest(
-                name,
-                KonanModelCreationRequest(
+                name=name,
+                description=None,
+                model_creation_request=KonanModelCreationRequest(
                     model_name,
                     docker_credentials,
                     docker_image,
+                    state=KonanModelState.Live,
                 )
             )
         )
         return deployment_creation_response
 
+    @deprecated.deprecated(
+        reason='Creating a Live Model alonside a Deployment will be removed in a future release',
+        version='1.4.0',
+        category=DeprecationWarning,
+    )
+    def create_deployment(
+        self,
+        name: str,
+        docker_credentials: Optional[KonanDockerCredentials],
+        docker_image: KonanDockerImage,
+        model_name: str = None,
+    ) -> KonanDeploymentCreationResponse:
+        """Call the create deployment function
+
+        :param name: name of the deployment to create
+        :type name: str
+        :param docker_credentials: credentials for the docker registry to use
+        :type docker_credentials: Optional[KonanDockerCredentials]
+        :param docker_image: docker image information
+        :type docker_image: KonanDockerImage
+        :param model_name: name of the live model to create, defaults to None
+        If left as None, will default to the name of the deployment
+        :type model_name: str, optional
+        :return: konan_deployment_creation_response
+        :rtype: KonanDeploymentCreationResponse
+        """
+        model_name = model_name or name
+
+        try:
+            return self._create_deployment(
+                name=name,
+                docker_credentials=docker_credentials,
+                docker_image=docker_image,
+                model_name=model_name,
+            )
+        except HTTPError as e:
+            # Heuristic for determining if the Konan API has been updated to use Projects
+            if e.response.status_code == 400:
+                # Retry request but with Project creation
+                deployment = self._create_project(name=name)
+                live_model = self.create_model(
+                    deployment_uuid=deployment.uuid,
+                    name=model_name,
+                    docker_credentials=docker_credentials,
+                    docker_image=docker_image,
+                    model_state=KonanModelState.Live
+                )
+                return KonanDeploymentCreationResponse(
+                    deployment=deployment,
+                    live_model=live_model,
+                    errors=[],
+                    container_logs='',
+                )
+            else:
+                raise e
+
     def create_model(
         self,
         deployment_uuid: str,
         name: str,
-        docker_credentials: KonanDockerCredentials,
+        docker_credentials: Optional[KonanDockerCredentials],
         docker_image: KonanDockerImage,
+        model_state: KonanModelState = KonanModelState.Challenger,
     ) -> KonanModel:
         """Call the create model function
 
@@ -128,9 +239,11 @@ class KonanSDK:
         :param name: name of the model to create
         :type name: str
         :param docker_credentials: credentials for the docker registry to use
-        :type docker_credentials: KonanDockerCredentials
+        :type docker_credentials: Optional[KonanDockerCredentials]
         :param docker_image: docker image information
         :type docker_image: KonanDockerImage
+        :param model_state: state in which to create the Model, defaults to KonanModelState.Challenger
+        :type model_state: KonanModelState, optional
         :return: konan_model
         :rtype: KonanModel
         """
@@ -149,6 +262,7 @@ class KonanSDK:
                 name,
                 docker_credentials,
                 docker_image,
+                state=model_state,
             )
         )
         return konan_model
@@ -434,3 +548,20 @@ class KonanSDK:
             user=self.auth.user
         ).request(None)
         return delete_deployment_result
+
+    def get_predictions(
+        self, deployment_uuid: str,
+        start_time: datetime.datetime, end_time: datetime.datetime,
+    ) -> Generator[List[KonanPrediction], None, None]:
+        # check user performed login
+        self.auth._post_login_checks()
+
+        # Check if access token is valid and retrieve a new one if needed
+        self.auth.auto_refresh_token()
+
+        predictions_generator = GetPredictionsEndpoint(
+            api_url=self.api_url,
+            deployment_uuid=deployment_uuid, user=self.auth.user
+        ).get_pages(request_object=KonanTimeWindow(start_time, end_time))
+
+        return predictions_generator
